@@ -34,8 +34,7 @@ def sum_primes(n):
 
 print """Usage: python sum_primes.py [ncpus]
     [ncpus] - the number of workers to run in parallel,
-    if omitted it will be set to the number of processors in the system
-"""
+    if omitted it will be set to the number of processors in the system"""
 
 # tuple of all parallel python servers to connect with
 ppservers = ()
