@@ -1,5 +1,5 @@
 # Parallel Python Software: http://www.parallelpython.com
-# Copyright (c) 2005-2008, Vitalii Vanovschi
+# Copyright (c) 2005-2009, Vitalii Vanovschi
 # All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -33,11 +33,9 @@ forums
 
 import os
 import thread
-import popen2
 import logging
 import inspect
 import sys
-import sha
 import types
 import time
 import atexit
@@ -46,15 +44,24 @@ import cPickle as pickle
 import pptransport
 import ppauto
 
-copyright = "Copyright (c) 2005-2008 Vitalii Vanovschi. All rights reserved"
-version = "1.5.6"
+copyright = "Copyright (c) 2005-2009 Vitalii Vanovschi. All rights reserved"
+version = "1.5.7"
 
 # reconnect persistent rworkers in 5 sec
 _RECONNECT_WAIT_TIME = 5
 
 # we need to have set even in Python 2.3
-if "set" not in globals():
-    from sets import Set as set
+try:
+    set
+except NameError:
+    from sets import Set as set 
+
+_USE_SUBPROCESS = False
+try:
+    import subprocess
+    _USE_SUBPROCESS = True
+except ImportError:
+    import popen2
 
 
 class _Task(object):
@@ -116,25 +123,55 @@ class _Task(object):
 class _Worker(object):
     """Local worker class
     """
+    command = "\"" + sys.executable + "\" -u \"" \
+            + os.path.dirname(os.path.abspath(__file__))\
+            + os.sep + "ppworker.py\""
 
-    def __init__(self):
+    if sys.platform.startswith("win"):
+        # workargound for windows
+        command = "\"" + command + "\""
+    else:
+        # do not show "Borken pipe message" at exit on unix/linux
+        command += " 2>/dev/null"
+
+    def __init__(self, restart_on_free, pickle_proto):
         """Initializes local worker"""
-        command = "\"" + sys.executable + "\" \"" \
-                + os.path.dirname(os.path.abspath(__file__))\
-                + os.sep + "ppworker.py\""
+        self.restart_on_free = restart_on_free
+        self.pickle_proto = pickle_proto
+        self.start()
 
-        if sys.platform.startswith("win"):
-            # workargound for windows
-            command = "\"" + command + "\""
+    def start(self):
+        """Starts local worker"""
+        if _USE_SUBPROCESS:
+            proc = subprocess.Popen(self.command, stdin=subprocess.PIPE, \
+                    stdout=subprocess.PIPE, stderr=subprocess.PIPE, \
+                    shell=True)
+            self.t = pptransport.CPipeTransport(proc.stdout, proc.stdin)
         else:
-            # do not show "Borken pipe message" at exit on unix/linux
-            command += " 2>/dev/null"
-
-        self.t = pptransport.CPipeTransport(*popen2.popen3(command)[:2])
+            self.t = pptransport.CPipeTransport(\
+                    *popen2.popen3(self.command)[:2])
 
         self.pid = int(self.t.receive())
+        self.t.send(str(self.pickle_proto))
         self.is_free = True
 
+    def stop(self):
+        """Stops local worker"""
+        self.is_free = False
+        self.t.close()
+
+    def restart(self):
+        """Restarts local worker"""
+        self.stop()
+        self.start()
+
+    def free(self):
+        """Frees local worker"""
+        if self.restart_on_free:
+            self.restart()
+        else:
+            self.is_free = True
+
 
 class _RWorker(pptransport.CSocketTransport):
     """Remote worker class
@@ -240,7 +277,8 @@ class Server(object):
     default_secret = "epo20pdosl;dksldkmm"
 
     def __init__(self, ncpus="autodetect", ppservers=(), secret=None,
-            loglevel=logging.WARNING, logstream=sys.stderr):
+            loglevel=logging.WARNING, logstream=sys.stderr,
+            restart=False, proto=0):
         """Creates Server instance
 
            ncpus - the number of worker processes to start on the local
@@ -253,6 +291,8 @@ class Server(object):
                    custom passphrase for all network connections.
            loglevel - logging level
            logstream - log stream destination
+           restart - wheather to restart worker process after each task completion
+           proto - protocol number for pickle module
 
            With ncpus = 1 all tasks are executed consequently
            For the best performance either use the default "autodetect" value
@@ -281,6 +321,8 @@ class Server(object):
         self.__accurate_stats = True
         self.autopp_list = {}
         self.__active_rworkers_list_lock = thread.allocate_lock()
+        self.__restart_on_free = restart
+        self.__pickle_proto = proto
 
         # add local directory and sys.path to PYTHONPATH
         pythondirs = [os.getcwd()] + sys.path
@@ -318,7 +360,7 @@ class Server(object):
                 raise TypeError("secret must be of a string type")
             self.secret = str(secret)
         elif hasattr(user, "pp_secret"):
-            secret = getattr(user, "pp_secret")
+            secret = user["pp_secret"]
             if not isinstance(secret, types.StringType):
                 raise TypeError("secret must be of a string type")
             self.secret = str(secret)
@@ -349,7 +391,8 @@ class Server(object):
 
         # perform some checks for frequent mistakes
         if self.__exiting:
-            raise "Cannot submit jobs: server instance has been destroyed"
+            raise RuntimeError("Cannot submit jobs: server"\
+                    " instance has been destroyed")
 
         if not isinstance(args, tuple):
             raise TypeError("args argument must be a tuple")
@@ -404,7 +447,7 @@ class Server(object):
                 depfuncs += (arg, )
 
         sfunc = self.__dumpsfunc((func, ) + depfuncs, modules)
-        sargs = pickle.dumps(args)
+        sargs = pickle.dumps(args, self.__pickle_proto)
 
         self.__queue_lock.acquire()
         self.__queue.append((task, sfunc, sargs))
@@ -446,7 +489,8 @@ class Server(object):
         if ncpus < 0:
             raise ValueError("ncpus must be an integer > 0")
         if ncpus > len(self.__workers):
-            self.__workers.extend([_Worker() for x in \
+            self.__workers.extend([_Worker(self.__restart_on_free, 
+                    self.__pickle_proto) for x in\
                     range(ncpus - len(self.__workers))])
         self.__stats["local"].ncpus = ncpus
         self.__ncpus = ncpus
@@ -494,7 +538,7 @@ class Server(object):
 
         if not self.__accurate_stats:
             print "WARNING: statistics provided above is not accurate" \
-                  "due to job rescheduling"
+                  " due to job rescheduling"
         print
 
     # all methods below are for internal use only
@@ -589,7 +633,8 @@ class Server(object):
         if hashs not in self.__sfuncHM:
             sources = [self.__get_source(func) for func in funcs]
             self.__sfuncHM[hashs] = pickle.dumps(
-                    (funcs[0].func_name, sources, modules))
+                    (funcs[0].func_name, sources, modules),
+                    self.__pickle_proto)
         return self.__sfuncHM[hashs]
 
     def __find_modules(self, prefix, dict):
@@ -617,7 +662,7 @@ class Server(object):
                         break
                 else:
                     self.__logger.error("There are no free workers left")
-                    raise "Error: No free workers"
+                    raise RuntimeError("Error: No free workers")
                 self.__add_to_active_tasks(1)
                 try:
                     self.__stats["local"].njobs += 1
@@ -721,7 +766,7 @@ class Server(object):
             else:
                 sys.excepthook(*sys.exc_info())
 
-        worker.is_free = True
+        worker.free()
 
         job.finalize(sresult)
 
@@ -743,10 +788,10 @@ class Server(object):
         self.__active_tasks += num
         self.__active_tasks_lock.release()
 
-    def __stat_add_time(self, node, time):
+    def __stat_add_time(self, node, time_add):
         """Updates total runtime on the node"""
         self.__stats_lock.acquire()
-        self.__stats[node].time += time
+        self.__stats[node].time += time_add
         self.__stats_lock.release()
 
     def __stat_add_job(self, node):
