@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # Parallel Python Software: http://www.parallelpython.com
-# Copyright (c) 2005-2008, Vitalii Vanovschi
+# Copyright (c) 2005-2009, Vitalii Vanovschi
 # All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -38,15 +38,24 @@ import socket
 import thread
 import random
 import string
-import sha
 import time
 import os
+
 import pptransport
 import ppauto
 from pp import Server
 
-copyright = "Copyright (c) 2005-2008 Vitalii Vanovschi. All rights reserved"
-version = "1.5.6"
+
+copyright = "Copyright (c) 2005-2009 Vitalii Vanovschi. All rights reserved"
+version = "1.5.7"
+
+# compartibility with Python 2.6
+try:
+    import hashlib
+    sha_new = hashlib.sha1
+except ImportError:
+    import sha
+    sha_new = sha.new
 
 
 class _NetworkServer(Server):
@@ -55,8 +64,10 @@ class _NetworkServer(Server):
 
     def __init__(self, ncpus="autodetect", interface="0.0.0.0",
                 broadcast="255.255.255.255", port=None, secret=None,
-                timeout=None, loglevel=logging.WARNING):
-        Server.__init__(self, ncpus, secret=secret, loglevel=loglevel)
+                timeout=None, loglevel=logging.WARNING, restart=False,
+                proto=0):
+        Server.__init__(self, ncpus, secret=secret, loglevel=loglevel,
+                restart=restart, proto=proto)
         self.host = interface
         self.bcast = broadcast
         if port is not None:
@@ -91,7 +102,7 @@ class _NetworkServer(Server):
                     time.sleep(self.timeout - idle_time)
                 else:
                     logging.debug("exiting ppserver due to timeout (no client"\
-                            "connections in last %i sec)", self.timeout)
+                            " connections in last %i sec)", self.timeout)
                     os._exit(0)
             else:
                 time.sleep(self.timeout)
@@ -128,7 +139,7 @@ class _NetworkServer(Server):
         srandom = "".join([random.choice(string.ascii_letters)
                 for i in xrange(16)])
         mysocket.send(srandom)
-        answer = sha.new(srandom+self.secret).hexdigest()
+        answer = sha_new(srandom+self.secret).hexdigest()
         cleintanswer = mysocket.receive()
         if answer != cleintanswer:
             logging.warning("Authentification failed, client host=%s, port=%i"
@@ -166,19 +177,101 @@ class _NetworkServer(Server):
     def broadcast(self):
         """Initiaates auto-discovery mechanism"""
         discover = ppauto.Discover(self)
-        thread.start_new_thread(discover.run, ((self.host, self.port), (self.bcast, self.port), ))
+        thread.start_new_thread(discover.run,
+                                ((self.host, self.port),
+                                 (self.bcast, self.port)),
+                               )
+
+
+def parse_config(file_loc):
+    """
+    Parses a config file in a very forgiving way.
+    """
+    # If we don't have configobj installed then let the user know and exit
+    try:
+        from configobj import ConfigObj
+    except ImportError, ie:
+        print >> sys.stderr, "ERROR: You must have configobj installed to use \
+configuration files. You can still use command line switches."
+        sys.exit(1)
+
+    if not os.access(file_loc, os.F_OK):
+        print >> sys.stderr, "ERROR: Can not access %s." % arg
+        sys.exit(1)
+
+    # Load the configuration file
+    config = ConfigObj(file_loc)
+    # try each config item and use the result if it exists. If it doesn't
+    # then simply pass and move along
+    try:
+        args['secret'] = config['general'].get('secret')
+    except:
+        pass
+
+    try:
+        autodiscovery = config['network'].as_bool('autodiscovery')
+    except:
+        pass
+
+    try:
+        args['interface'] = config['network'].get('interface',
+                                                  default="0.0.0.0")
+    except:
+        pass
+
+    try:
+        args['broadcast'] = config['network'].get('broadcast')
+    except:
+        pass
+
+    try:
+        args['port'] = config['network'].as_int('port')
+    except:
+        pass
+
+    try:
+        args['loglevel'] = config['general'].as_bool('debug')
+    except:
+        pass
+
+    try:
+        args['ncpus'] = config['general'].as_int('workers')
+    except:
+        pass
+
+    try:
+        args['proto'] = config['general'].as_int('proto')
+    except:
+        pass
+
+    try:
+        args['restart'] = config['general'].as_bool('restart')
+    except:
+        pass
+
+    try:
+        args['timeout'] = config['network'].as_int('timeout')
+    except:
+        pass
+    # Return a tuple of the args dict and autodiscovery variable
+    return args, autodiscovery
 
 
 def print_usage():
     """Prints help"""
-    print "Parallel Python Network Server (pp-"+version+")"
-    print "Usage: ppserver.py [-hda] [-i interface] [-b broadcast] "\
-          "[-p port] [-w nworkers] [-s secret] [-t seconds]"
+    print "Parallel Python Network Server (pp-" + version + ")"
+    print "Usage: ppserver.py [-hdar] [-n proto] [-c config_path]"\
+            " [-i interface] [-b broadcast] [-p port] [-w nworkers]"\
+            " [-s secret] [-t seconds]"
     print
     print "Options: "
     print "-h                 : this help message"
     print "-d                 : debug"
     print "-a                 : enable auto-discovery service"
+    print "-r                 : restart worker process after each"\
+            " task completion"
+    print "-n proto           : protocol number for pickle module"
+    print "-c path            : path to config file"
     print "-i interface       : interface to listen"
     print "-b broadcast       : broadcast address for auto-discovery service"
     print "-p port            : port to listen"
@@ -197,7 +290,8 @@ def print_usage():
 
 if __name__ == "__main__":
     try:
-        opts, args = getopt.getopt(sys.argv[1:], "hdab:i:p:w:s:t:", ["help"])
+        opts, args = getopt.getopt(sys.argv[1:], 
+            "hdarn:c:b:i:p:w:s:t:", ["help"])
     except getopt.GetoptError:
         print_usage()
         sys.exit(1)
@@ -209,6 +303,8 @@ if __name__ == "__main__":
         if opt in ("-h", "--help"):
             print_usage()
             sys.exit()
+        elif opt == "-c":
+            args, autodiscovery = parse_config(arg)
         elif opt == "-d":
             args["loglevel"] = logging.DEBUG
         elif opt == "-i":
@@ -221,8 +317,12 @@ if __name__ == "__main__":
             args["ncpus"] = int(arg)
         elif opt == "-a":
             autodiscovery = True
+        elif opt == "-r":
+            args["restart"] = True
         elif opt == "-b":
             args["broadcast"] = arg
+        elif opt == "-n":
+            args["proto"] = int(arg)
         elif opt == "-t":
             args["timeout"] = int(arg)
 
