@@ -1,5 +1,5 @@
 # Parallel Python Software: http://www.parallelpython.com
-# Copyright (c) 2005-2008, Vitalii Vanovschi
+# Copyright (c) 2005-2009, Vitalii Vanovschi
 # All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -29,15 +29,24 @@ Parallel Python Software, PP Transport
 http://www.parallelpython.com - updates, documentation, examples and support
 forums
 """
-import md5
 import os
 import struct
-import sha
 import socket
 import logging
 
-copyright = "Copyright (c) 2005-2008 Vitalii Vanovschi. All rights reserved"
-version = "1.5.6"
+copyright = "Copyright (c) 2005-2009 Vitalii Vanovschi. All rights reserved"
+version = "1.5.7"
+
+# compartibility with Python 2.6
+try:
+    import hashlib
+    sha_new = hashlib.sha1
+    md5_new = hashlib.md5
+except ImportError:
+    import sha
+    import md5
+    sha_new = sha.new
+    md5_new = md5.new
 
 
 class Transport(object):
@@ -46,7 +55,7 @@ class Transport(object):
         raise NotImplemented("abstact function 'send' must be implemented "\
                 "in a subclass")
 
-    def receive(self):
+    def receive(self, preprocess=None):
         raise NotImplemented("abstact function 'receive' must be implemented "\
                 "in a subclass")
 
@@ -58,7 +67,7 @@ class Transport(object):
             logging.error("Please install the same version of PP on all nodes")
             return False
         srandom = self.receive()
-        answer = sha.new(srandom+secret).hexdigest()
+        answer = sha_new(srandom+secret).hexdigest()
         self.send(answer)
         response = self.receive()
         if response == "OK":
@@ -69,7 +78,7 @@ class Transport(object):
     def close(self):
         pass
 
-    def _connect(self):
+    def _connect(self, host, port):
         pass
 
 
@@ -79,7 +88,7 @@ class CTransport(Transport):
     rcache = {}
 
     def hash(self, msg):
-        return md5.new(msg).hexdigest()
+        return md5_new(msg).hexdigest()
 
     def csend(self, msg):
         hash1 = self.hash(msg)
@@ -113,19 +122,14 @@ class PipeTransport(Transport):
                     "must be file objects")
 
     def send(self, msg):
-        self.w.write(str(len(msg))+"\n")
+        self.w.write(struct.pack("!Q", len(msg)))
+        self.w.flush()
         self.w.write(msg)
         self.w.flush()
 
     def receive(self, preprocess=None):
-        s = self.r.readline()
-        try:
-            msg_len = int(s)
-        except ValueError:
-            if self.exiting:
-                return
-            print "Literal for int() is: \"", s, "\""
-            raise
+        size_packed = self.r.read(struct.calcsize("!Q"))
+        msg_len = struct.unpack("!Q", size_packed)[0]
         msg = self.r.read(msg_len)
         return map(preprocess, (msg, ))[0]
 
@@ -161,7 +165,7 @@ class SocketTransport(Transport):
                 raise RuntimeError("Socket connection is broken")
             s_size += p_size
 
-    def receive(self):
+    def receive(self, preprocess=None):
         e_size = struct.calcsize("!Q")
         r_size = 0
         data = ""
