@@ -1,5 +1,5 @@
 # Parallel Python Software: http://www.parallelpython.com
-# Copyright (c) 2005-2008, Vitalii Vanovschi
+# Copyright (c) 2005-2009, Vitalii Vanovschi
 # All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -35,8 +35,8 @@ import StringIO
 import cPickle as pickle
 import pptransport
 
-copyright = "Copyright (c) 2005-2008 Vitalii Vanovschi. All rights reserved"
-version = "1.5.6"
+copyright = "Copyright (c) 2005-2009 Vitalii Vanovschi. All rights reserved"
+version = "1.5.7"
 
 
 def import_module(name):
@@ -70,6 +70,7 @@ class _WorkerProcess(object):
         sys.stderr = self.sout
         self.t = pptransport.CPipeTransport(sys.stdin, sys.__stdout__)
         self.t.send(str(os.getpid()))
+        self.pickle_proto = int(self.t.receive())
 
     def run(self):
         try:
@@ -99,7 +100,8 @@ class _WorkerProcess(object):
                     sys.excepthook(*sys.exc_info())
                     __result = None
 
-                __sresult = pickle.dumps((__result, self.sout.getvalue()))
+                __sresult = pickle.dumps((__result, self.sout.getvalue()),
+                        self.pickle_proto)
 
                 self.t.send(__sresult)
                 self.sout.truncate(0)
@@ -107,7 +109,8 @@ class _WorkerProcess(object):
             print "Fatal error has occured during the function execution"
             sys.excepthook(*sys.exc_info())
             __result = None
-            __sresult = pickle.dumps((__result, self.sout.getvalue()))
+            __sresult = pickle.dumps((__result, self.sout.getvalue()),
+                    self.pickle_proto)
             self.t.send(__sresult)
 
 
